@@ -1,10 +1,11 @@
 import os
+import warnings
 
 import eventlet
-from eventlet import debug, event
+from eventlet import convenience, debug
 from eventlet.green import socket
-from eventlet.support import six
 import tests
+import tests.mock
 
 
 certificate_file = os.path.join(os.path.dirname(__file__), 'test_server.crt')
@@ -13,11 +14,11 @@ private_key_file = os.path.join(os.path.dirname(__file__), 'test_server.key')
 
 class TestServe(tests.LimitedTestCase):
     def setUp(self):
-        super(TestServe, self).setUp()
+        super().setUp()
         debug.hub_exceptions(False)
 
     def tearDown(self):
-        super(TestServe, self).tearDown()
+        super().tearDown()
         debug.hub_exceptions(True)
 
     def test_exiting_server(self):
@@ -66,7 +67,7 @@ class TestServe(tests.LimitedTestCase):
             hits[0] += 1
         l = eventlet.listen(('localhost', 0))
         gt = eventlet.spawn(eventlet.serve, l, counter)
-        for i in six.moves.range(100):
+        for i in range(100):
             client = eventlet.connect(('localhost', l.getsockname()[1]))
             self.assertFalse(client.recv(100))
         gt.kill()
@@ -90,7 +91,7 @@ class TestServe(tests.LimitedTestCase):
         gt.wait()
 
     def test_concurrency(self):
-        evt = event.Event()
+        evt = eventlet.Event()
 
         def waiter(sock, addr):
             sock.sendall(b'hi')
@@ -128,18 +129,67 @@ class TestServe(tests.LimitedTestCase):
         client.sendall(b"echo")
         self.assertEqual(b"echo", client.recv(1024))
 
-    def test_socket_reuse(self):
+
+def test_socket_reuse():
+    # pick a free port with bind to 0 - without SO_REUSEPORT
+    # then close it and try to bind to same port with SO_REUSEPORT
+    # loop helps in case something else used the chosen port before second bind
+    addr = None
+    errors = []
+    for _ in range(5):
         lsock1 = eventlet.listen(('localhost', 0))
-        port = lsock1.getsockname()[1]
-
-        if hasattr(socket, 'SO_REUSEPORT'):
-            lsock2 = eventlet.listen(('localhost', port))
-        else:
-            try:
-                lsock2 = eventlet.listen(('localhost', port))
-                assert lsock2
-                lsock2.close()
-            except socket.error:
-                pass
-
+        addr = lsock1.getsockname()
         lsock1.close()
+        try:
+            lsock1 = eventlet.listen(addr)
+        except OSError as e:
+            errors.append(e)
+            continue
+        break
+    else:
+        assert False, errors
+
+    if hasattr(socket, 'SO_REUSEPORT'):
+        lsock2 = eventlet.listen(addr)
+    else:
+        try:
+            lsock2 = eventlet.listen(addr)
+            assert lsock2
+            lsock2.close()
+        except OSError:
+            pass
+
+    lsock1.close()
+
+
+def test_reuse_random_port_warning():
+    with warnings.catch_warnings(record=True) as w:
+        eventlet.listen(('localhost', 0), reuse_port=True).close()
+        assert len(w) == 1
+        assert issubclass(w[0].category, convenience.ReuseRandomPortWarning)
+
+
+@tests.skip_unless(hasattr(socket, 'SO_REUSEPORT'))
+def test_reuseport_oserror():
+    # https://github.com/eventlet/eventlet/issues/380
+    # https://github.com/eventlet/eventlet/issues/418
+    err22 = OSError(22, 'Invalid argument')
+
+    sock1 = eventlet.listen(('localhost', 0))
+    addr = sock1.getsockname()
+    sock1.close()
+
+    original_socket_init = socket.socket.__init__
+
+    def patched(self, *a, **kw):
+        original_socket_init(self, *a, **kw)
+        self.setsockopt = tests.mock.Mock(side_effect=err22)
+
+    with warnings.catch_warnings(record=True) as w:
+        try:
+            socket.socket.__init__ = patched
+            eventlet.listen(addr, reuse_addr=False, reuse_port=True).close()
+        finally:
+            socket.socket.__init__ = original_socket_init
+        assert len(w) == 1
+        assert issubclass(w[0].category, convenience.ReusePortUnavailableWarning)
