@@ -76,16 +76,7 @@ except ImportError:
             return f
         return inner
 else:
-    if sys.version_info[:2] >= (3, 2):
-        wraps = original_wraps
-    else:
-        def wraps(func):
-            def inner(f):
-                f = original_wraps(func)(f)
-                wrapped = getattr(func, '__wrapped__', func)
-                f.__wrapped__ = wrapped
-                return f
-            return inner
+    wraps = original_wraps
 
 try:
     unicode
@@ -160,7 +151,7 @@ def _is_exception(obj):
     )
 
 
-class _slotted(object):
+class _slotted:
     __slots__ = ['a']
 
 
@@ -363,7 +354,7 @@ def _is_magic(name):
     return '__%s__' % name[2:-2] == name
 
 
-class _SentinelObject(object):
+class _SentinelObject:
     "A unique, named, sentinel object."
 
     def __init__(self, name):
@@ -373,7 +364,7 @@ class _SentinelObject(object):
         return 'sentinel.%s' % self.name
 
 
-class _Sentinel(object):
+class _Sentinel:
     """Access attributes to return a named object, usable as a sentinel."""
 
     def __init__(self):
@@ -408,13 +399,11 @@ ClassTypes = (type,)
 if not inPy3k:
     ClassTypes = (type, ClassType)
 
-_allowed_names = set(
-    [
+_allowed_names = {
         'return_value', '_mock_return_value', 'side_effect',
         '_mock_side_effect', '_mock_parent', '_mock_new_parent',
         '_mock_name', '_mock_new_name'
-    ]
-)
+}
 
 
 def _delegating_property(name):
@@ -482,7 +471,7 @@ def _check_and_set_parent(parent, value, name, new_name):
     return True
 
 
-class Base(object):
+class Base:
     _mock_return_value = DEFAULT
     _mock_side_effect = None
 
@@ -1104,7 +1093,7 @@ def _is_started(patcher):
     return hasattr(patcher, 'is_local')
 
 
-class _patch(object):
+class _patch:
 
     attribute_name = None
     _active_patches = set()
@@ -1554,7 +1543,7 @@ def patch(
     )
 
 
-class _patch_dict(object):
+class _patch_dict:
     """
     Patch a dictionary, or dictionary like object, and restore the dictionary
     to its original state after the test.
@@ -1712,13 +1701,13 @@ else:
 # (as they are metaclass methods)
 # __del__ is not supported at all as it causes problems if it exists
 
-_non_defaults = set('__%s__' % method for method in [
+_non_defaults = {'__%s__' % method for method in [
     'cmp', 'getslice', 'setslice', 'coerce', 'subclasses',
     'format', 'get', 'set', 'delete', 'reversed',
     'missing', 'reduce', 'reduce_ex', 'getinitargs',
     'getnewargs', 'getstate', 'setstate', 'getformat',
     'setformat', 'repr', 'dir'
-])
+]}
 
 
 def _get_method(name, func):
@@ -1730,19 +1719,19 @@ def _get_method(name, func):
     return method
 
 
-_magics = set(
+_magics = {
     '__%s__' % method for method in
     ' '.join([magic_methods, numerics, inplace, right, extra]).split()
-)
+}
 
 _all_magics = _magics | _non_defaults
 
-_unsupported_magics = set([
+_unsupported_magics = {
     '__getattr__', '__setattr__',
     '__init__', '__new__', '__prepare__'
     '__instancecheck__', '__subclasscheck__',
     '__del__'
-])
+}
 
 _calculate_return_value = {
     '__hash__': lambda self: object.__hash__(self),
@@ -1827,7 +1816,7 @@ def _set_return_value(mock, method, name):
         method.side_effect = side_effector(mock)
 
 
-class MagicMixin(object):
+class MagicMixin:
     def __init__(self, *args, **kw):
         _super(MagicMixin, self).__init__(*args, **kw)
         self._mock_set_magics()
@@ -1889,7 +1878,7 @@ class MagicMock(MagicMixin, Mock):
         self._mock_set_magics()
 
 
-class MagicProxy(object):
+class MagicProxy:
     def __init__(self, name, parent):
         self.name = name
         self.parent = parent
@@ -1911,7 +1900,7 @@ class MagicProxy(object):
         return self.create_mock()
 
 
-class _ANY(object):
+class _ANY:
     "A helper object that compares equal to everything."
 
     def __eq__(self, other):
@@ -2250,7 +2239,7 @@ def _get_class(obj):
         return type(obj)
 
 
-class _SpecState(object):
+class _SpecState:
 
     def __init__(self, spec, spec_set=False, parent=None,
                  name=None, ids=None, instance=False):
@@ -2271,7 +2260,7 @@ FunctionTypes = (
     type(_ANY.__eq__),
 )
 
-FunctionAttributes = set([
+FunctionAttributes = {
     'func_closure',
     'func_code',
     'func_defaults',
@@ -2279,7 +2268,7 @@ FunctionAttributes = set([
     'func_doc',
     'func_globals',
     'func_name',
-])
+}
 
 
 file_spec = None
