@@ -3,7 +3,7 @@ import shutil
 import sys
 import tempfile
 
-from eventlet.support import six
+import pytest
 import tests
 
 
@@ -33,12 +33,12 @@ class ProcessBase(tests.LimitedTestCase):
     TEST_TIMEOUT = 3  # starting processes is time-consuming
 
     def setUp(self):
-        super(ProcessBase, self).setUp()
+        super().setUp()
         self._saved_syspath = sys.path
         self.tempdir = tempfile.mkdtemp('_patcher_test')
 
     def tearDown(self):
-        super(ProcessBase, self).tearDown()
+        super().tearDown()
         sys.path = self._saved_syspath
         shutil.rmtree(self.tempdir)
 
@@ -52,11 +52,8 @@ class ProcessBase(tests.LimitedTestCase):
     def launch_subprocess(self, filename):
         path = os.path.join(self.tempdir, filename)
         output = tests.run_python(path)
-        if six.PY3:
-            output = output.decode('utf-8')
-            separator = '\n'
-        else:
-            separator = b'\n'
+        output = output.decode('utf-8')
+        separator = '\n'
         lines = output.split(separator)
         return output, lines
 
@@ -82,26 +79,13 @@ class ImportPatched(ProcessBase):
         assert 'eventlet.green.urllib' in lines[2], repr(output)
         assert 'eventlet.green.httplib' not in lines[2], repr(output)
 
-    def test_import_patched_defaults(self):
-        self.write_to_tempfile("base", """
-import socket
-try:
-    import urllib.request as urllib
-except ImportError:
-    import urllib
-print("base {0} {1}".format(socket, urllib))""")
 
-        new_mod = """
-from eventlet import patcher
-base = patcher.import_patched('base')
-print("newmod {0} {1} {2}".format(base, base.socket, base.urllib.socket.socket))
-"""
-        self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod.py')
-        assert lines[0].startswith('base'), repr(output)
-        assert lines[1].startswith('newmod'), repr(output)
-        assert 'eventlet.green.socket' in lines[1], repr(output)
-        assert 'GreenSocket' in lines[1], repr(output)
+def test_import_patched_defaults():
+    tests.run_isolated('patcher_import_patched_defaults.py')
+
+
+def test_import_patched_handles_sub_modules():
+    tests.run_isolated('test_sub_module_in_import_patched/test.py')
 
 
 class MonkeyPatch(ProcessBase):
@@ -226,8 +210,7 @@ def test_monkey_patch_threading():
     tickcount = [0]
 
     def tick():
-        from eventlet.support import six
-        for i in six.moves.range(1000):
+        for i in range(1000):
             tickcount[0] += 1
             eventlet.sleep()
 
@@ -246,7 +229,6 @@ def test_monkey_patch_threading():
 class Tpool(ProcessBase):
     TEST_TIMEOUT = 3
 
-    @tests.skip_with_pyevent
     def test_simple(self):
         new_mod = """
 import eventlet
@@ -264,7 +246,6 @@ tpool.killall()
         assert '2' in lines[0], repr(output)
         assert '3' in lines[1], repr(output)
 
-    @tests.skip_with_pyevent
     def test_unpatched_thread(self):
         new_mod = """import eventlet
 eventlet.monkey_patch(time=False, thread=False)
@@ -277,7 +258,6 @@ import time
         output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 2, lines)
 
-    @tests.skip_with_pyevent
     def test_patched_thread(self):
         new_mod = """import eventlet
 eventlet.monkey_patch(time=False, thread=True)
@@ -329,23 +309,6 @@ print(len(_threading._active))
         assert lines[1] == '1', lines
         assert lines[2] == '1', lines
 
-    def test_threading(self):
-        new_mod = """import eventlet
-eventlet.monkey_patch()
-import threading
-def test():
-    print(repr(threading.currentThread()))
-t = threading.Thread(target=test)
-t.start()
-t.join()
-print(len(threading._active))
-"""
-        self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod.py')
-        self.assertEqual(len(lines), 3, "\n".join(lines))
-        assert lines[0].startswith('<_MainThread'), lines[0]
-        self.assertEqual(lines[1], "1", lines[1])
-
     def test_tpool(self):
         new_mod = """import eventlet
 eventlet.monkey_patch()
@@ -510,6 +473,15 @@ def test_patcher_existing_locks_locked():
     tests.run_isolated('patcher_existing_locks_locked.py')
 
 
+@tests.skip_if_CRLock_exist
+def test_patcher_existing_locks_unlocked():
+    tests.run_isolated('patcher_existing_locks_unlocked.py')
+
+
+def test_patcher_existing_logging_module_lock():
+    tests.run_isolated('patcher_existing_logging_module_lock.py')
+
+
 def test_importlib_lock():
     tests.run_isolated('patcher_importlib_lock.py')
 
@@ -532,3 +504,39 @@ def test_blocking_select_methods_are_deleted():
 
 def test_regular_file_readall():
     tests.run_isolated('regular_file_readall.py')
+
+
+def test_threading_current():
+    tests.run_isolated('patcher_threading_current.py')
+
+
+def test_threadpoolexecutor():
+    tests.run_isolated('patcher_threadpoolexecutor.py')
+
+
+@pytest.mark.skipif(
+    not sys.platform.startswith("linux"),
+    reason="fork() doesn't work well on macOS, and definitely doesn't work on Windows"
+)
+def test_fork_after_monkey_patch():
+    tests.run_isolated('patcher_fork_after_monkey_patch.py')
+
+
+def test_builtin():
+    tests.run_isolated('patcher_builtin.py')
+
+
+def test_open_kwargs():
+    tests.run_isolated("patcher_open_kwargs.py")
+
+
+def test_patcher_existing_locks():
+    tests.run_isolated("patcher_existing_locks_preexisting.py")
+
+
+def test_patcher_existing_locks_exception():
+    tests.run_isolated("patcher_existing_locks_exception.py")
+
+
+def test_patcher_threading_subclass_done():
+    tests.run_isolated("patcher_threading_subclass_done.py")
